[ORG 0x7c00]
[BITS 16] 
    mov ax, 0
    mov ss, ax              ; nastavime vrchol zasobniku
    mov sp, 0x7c00          ; segment i offset
 
    mov si, mode16_txt
    call print
 
    cli                     ; zablokujeme preruseni
    lgdt [gdt_desc]         ; nacteme do GDTR deskriptor GDT
    
    mov eax, cr0            ; v ridicim registru CR0 nastavime
    or eax, 1               ; bit indikujici chraneny mod
    mov cr0, eax          
 
    jmp 0x08:main32         ; skok do 32bitoveho segmentu (selektor 0x08)
 
    mov si, mode16_fail
    call print
  
stop:
    jmp stop               ; vstoupime do nekonecne smycky 

;
; pomocna funkce pro vypis na terminal
; adresa retezce je predana v si
;
print:
    mov ah, 0x0e           ; identifikator vystupni operace

print_char:
    mov al, [si]           ; precteme jeden znak
    cmp al, 0              ; pokud je \0,
    jz print_done          ; skoncime

    int 0x10               ; vypiseme znak
    add si, 1              ; a pokracujeme dalsim znakem
    jmp print_char
print_done:
    ret

;
; 32bitovy kod
;

[BITS 32]
main32:
    mov ax, 0x10
    mov ds, ax
    mov esi, mode32_txt
    mov edi, 0xb8000

print_char32:
    mov al, [esi]
    cmp al, 0
    je stop32

    mov [edi], al
    mov byte [edi + 1], 0x1f

    add esi, 1
    add edi, 2
    jmp print_char32

stop32:
   jmp stop32

;
; GDT
;

gdt:
gdt_null:            ; povinny null-zaznam v GDT
    dd 0x0
    dd 0x0

gdt_cs:              ; descriptor kodoveho segmentu
    dw 0xffff        ; limit (bity 0-15)
    dw 0x0000        ; base (bity 0-15)
    db 0x00          ; base (bity 16-23)
    db 10011010b     ; 1: present; 00: ring0; 1: not-system-seg.; 1: code-seg.; 0: not-conforming; 1: read/write; 0: accessed
    db 11001111b     ; 1: granularity = pages; 1: 32bit mode; 00: reserved; 1111: limit (bity 16-19)
    db 0x00          ; base (bity 24-31)

gdt_ds:              ; descriptor datoveho segmentu
    dw 0xffff        ; limit (bity 0-15)
    dw 0x0000        ; base (bity 0-15)
    db 0x00          ; base (bity 16-23)
    db 10010010b     ; 1: present; 00: ring0; 1: not-system-seg.; 0: data-seg.; 0: grows up; 1: read/write; 0: accessed
    db 11001111b     ; 1: granularity = pages; 1: 32bit mode; 00: reserved; 1111: limit (bity 16-19)
    db 0x00          ; base (bity 24-31)
gdt_end:

gdt_desc:            ; descriptor tabulky GDT
    dw gdt_end - gdt - 1      ; velikost tabulky - 1
    dd gdt           ; adresa GDT


;
; podpurna data 
;

mode16_txt:
    db '16bit mode', 13, 10, 0

mode32_txt:
    db '*** 32bit mode ***', 0

mode16_fail:
    db 'switch failure', 13, 10, 0

    times 510 -($-$$) db 0 ; vyplnime zbyvajici prostor 0
    dw 0xaa55              ; signatura MBR
